#include"inout.h"
#include<stdlib.h>
#define BOOL int
#define TRUE 1
#define FALSE 0
// Alias à utilser partout pour éviter de mettre 6 et 7 partout dans le code
#define NBL 3 // Nombre de ligne du puissance 4
#define NBC 3 // Nombre de colonnes du puissance 4
#define COCHEVIDE '_'
#define COCHEX 'X'
#define COCHEO 'O'

// MANIPULATION DES COORDONNÉES ET NUMÉROS DE CASES

// On utilise parfois les numéro de case (de 0 à 8) et parfois les
// [x][y], du coup on a besoin de traduction entre les deux numéroations

// Retourne le numéro la ligne d'une case
int getL(int n){
  return n/NBC;
}

// Retourne le numéro de la colonne d'une case
int getC(int n){
  return n%NBC;
}

// Retourne le numéro de la case [i][j]
int getPos(int l, int c){
  return l*NBC+c;
}

// MANIPULATION DE LA GRILLE

// Retourne le caractère de la case n de la grille t
char getCase(char t[][NBC], int n){
  return t[getL(n)][getC(n)];
}

// remplace la case n de la grille t par le caractère coche
void setCase(char t[][NBC], int n, char coche){
  t[getL(n)][getC(n)] = coche;
}

/* Affiche la grille dans le terminal: NBL lignes de NBC caractères */
void afficheGrille(char tc[][NBC]){
  int i,j;
  for(i = 0 ; i < NBL ; i = i + 1){
      for(j = 0 ; j < NBC ; j = j + 1){
        ecrireChar(tc[i][j]);
      }
      ecrireSautDeLigne();
  }
}

// TEST DE VICTOIRE

/* Retourne vrai si c1 c2 et c3 contiennent le même caractère,
   différent de COCHEVIDE, et faux sinon */
BOOL caseIdentiquesNonVide(char t[][NBC], int c1, int c2, int c3){
  return getCase(t,c1) != COCHEVIDE
    && getCase(t,c1) == getCase(t,c2)
    && getCase(t,c2) == getCase(t,c3);
}

/* Retourne vrai si il existe une ligne, une colonne ou une diagonale
   contenant le même caractère, différent de COCHEVIDE. Attention ceci ne
   marche que pour les grilles 3x3. Pour rendre le test compatible
   avec d'autres taille de grille il faudrait être capable de ne
   tester que les combinaisons contenant la dernière case cochée. */
BOOL estGagnante(char t[][NBC]){
  return
       caseIdentiquesNonVide(t,0,1,2)
    || caseIdentiquesNonVide(t,3,4,5)
    || caseIdentiquesNonVide(t,6,7,8)
    || caseIdentiquesNonVide(t,0,3,6)
    || caseIdentiquesNonVide(t,1,4,7)
    || caseIdentiquesNonVide(t,2,5,8)
    || caseIdentiquesNonVide(t,0,4,8)
    || caseIdentiquesNonVide(t,2,4,6)
    ;
}

// LIRE UN COUP DE JOUEUR JUSQU'À UNE RÉPONSE CORRECTE

/* Demande une case vide. Redemande jusqu'à obtenir une réponse
   correcte. */
int safeLireInt(char tc[][NBC]){
  int c;
  BOOL OK = FALSE;
  while(!OK){
    printf("Tapez le numéro d'une case");
    c = lireIntDefaut(0)-1;
    if(c!=-1 && c>=0 && c < NBL*NBL && getCase(tc,c) == COCHEVIDE){
      OK=TRUE;
    } else {
      printf("ERREUR! Vous devez désigner un numéro de case, vide de surcroît.\n");
    }
  }
  return c;
}

// PROGRAMME PRINCIPAL

int main() {
  char t[NBL][NBC];
  int i,j;
  for(i = 0 ; i < NBL ; i = i + 1){
      for(j = 0 ; j < NBC ; j = j + 1){
        t[i][j] = COCHEVIDE;
      }
  }

  int casecochee;
  BOOL gagne=FALSE;
  int nbCout=0; // Pour savoir quand toutes les cases sont cochées.
  char coche = COCHEX;

  while(nbCout< NBL*NBC && !gagne) {
    printf("\n");
    afficheGrille(t);
    casecochee= safeLireInt(t);
    setCase(t,casecochee,coche);
    gagne=estGagnante(t);
    if (!gagne) {
      if(coche==COCHEX) { coche = COCHEO; }
      else { coche = COCHEX; }
    }
    nbCout = nbCout + 1;
  }
  // On sort de la boucle soit parce-qu'il y a victoire, soit
  // parce-que toutes les cases sont cochée. On adapte le message de
  // fin de partie en fonction:
  if(gagne){
    printf ("**** Le joueur %c a gagné! ****\n",coche);
  } else {
    printf("\n**** Partie nulle ****\n");
  }
  // Affichage de la grille finale
  afficheGrille(t);

}
