#!/bin/bash

USAGE="
testPartie.sh partieXXX.txt partieXXX.correction [nbLignes]

 execute le programme puissance4 avec la liste de coup stockée dans
 partieXXX.txt puis vérifie que les motifs listé dans
 partieXXX.correction apparaissent vers la fin du fichier.

 L'argument optionel nbLignes permet de spécifier combien de lignes
 considérer à la fin du fichier (par défaut 30).

 [-h]  Affiche cette documentation.
 [-v]  Affiche des informations supplémentaires pendant le test
 [-exec fichier] utilise l'exécutable fichier à la place de puissance4
"

# le programme à appeler. Peut être changé via l'option -exec
EXEC=../puissance4

# peut être changé via l'option -v
verbose=no

# Détection des options
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h)
        echo "$USAGE"
        exit
        ;;
    -exec)
        shift
        EXEC=$1
        shift
        ;;
    -v)
        verbose=yes
        shift
        ;;
    *)    # unknown option
        POSITIONAL+=("$1") # save it in an array for later
        shift # past argument
        ;;
esac
done

set -- "${POSITIONAL[@]}" # restore positional parameters (i.e.
                          # parameters that were not recognized by the
                          # previous code.)

echo "# TESTING \`$1\` vs \`$2\`"
echo "## COMMANDE (lancé depuis le répertoire \`$PWD\`): "
echo "\`$0 $*\`"

OTHEROPTIONS=$*

FICHIERCOUPS=$1
FICHIERMOTIFS=$2
# Argument $3 optionnel: Nombre de lignes à considérer 
NBLIGNES=$3
if [ "$NBLIGNES" == "" ];
then
    NBLIGNES=30
fi

ERREURMSG=OOOPS
OKMSG=OOOK
# Les mots et caractère utilisés par le script ne doivent pas
# apparaître dans la trace du jeu. On lance le jeu et on vérifie que
# les caractères spéciaux n'apparaissent pas dans la sortie du
# programme (les dernières $NBLIGNES lignes uniquement). Si ils
# apparaissent on plante tout de suite.
SANITY="@\|#\|þ\\|OOPS\\|OK\\|$ERREURMSG\\|OKMSG"

echoverbose() {
    if [ "$verbose" = "yes" ] ;
    then echo $*
    fi    
}

trace=$($EXEC < $FICHIERCOUPS | tail -$NBLIGNES)
echo "${trace}" | grep "${SANITY}"  > /dev/null
sanitycheck=$?
if [ "$sanitycheck" == "0" ];
then
    echo "******  SANITY CHECK FAILED ******"
    echo "${trace}" | grep "${SANITY}"
    exit 1;
fi

# On remplace les lsaut de ligne par @, on fera de même pour les
# motifs avant de comparer
trace=$(echo "${trace}" | tr '\n' "@" )

# Lecture du fichier contenant les motifs et fabrication de la listes
# des motifs. le résultat $motifs est une chaine contenant un test par
# ligne, avec les newlines remplacés par des @. Les lignes du test
# commençant par # sont ignorées. Les sauts de ligne à la fin du
# fichier aussi
motifs=$(cat "$FICHIERMOTIFS" | sed '/^#/d' | tr '\n' '@' | sed -Ee 's/@*$//g' | sed -Ee 's/^@*//g' | sed -Ee 's/@@+/þ/g' | tr 'þ' '\n' )

echoverbose ""
echoverbose "####### DEBUG: TRACE ANALYSÉE: {${trace}}"
# On vérifie que chaque motif apparaît bien dans la trace, on affiche
# la trace si le test est un échec
while IFS= read -r line
do
    if [ "${line}" != "" ];
    then
        echoverbose ""
        echoverbose "####### DEBUG MOTIF: {${line}}"
        echo "${trace}" | grep "${line}" > /dev/null
        success=$?
        if [ "$success" == "0" ] ;
        then
            echo
            echo "## $OKMSG:"
            echo "\`\`\`"
            echo "${line}" | tr '@' '\n'
            echo "\`\`\`"
        else
            echo
            echo "## _**$ERREURMSG**_:"
            echo motif attendu:
            echo "\`\`\`"
            echo "${line}" | tr '@' '\n'
            echo "\`\`\`"
            echo résultat obtenu:
            echo "\`\`\`"
            echo "$trace" | tr '@' '\n'
            echo "\`\`\`"
        fi
    fi
done <<< "$motifs"

